package org.qatools.remote;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.qatools.util.ListUtil;
import org.qatools.util.QAToolsUtil;
import org.qatools.util.SshUtil;

public abstract class RemoteFileUtil extends QAToolsUtil {
    private static Logger sLogger = Logger.getLogger(RemoteFileUtil.class);

    
	// -------------------------------------------------------------
	// ----- doesFileExist -----------------------------------------
	// -------------------------------------------------------------

    /**
     * 
     */
    public static boolean doesExist(RemoteFile rf) {
    	String mCount = SshUtil.runCommandGetResult(rf, "ls " + rf.getDir() + rf.getFilename() + " | wc -l");
    	if ("1".equals(mCount)) {
    		return true;
    	}
    	return false; 
    }
	
	// -------------------------------------------------------------
	// ----- doesExist ---------------------------------------------
	// -------------------------------------------------------------

    public static List<String> getLines(RemoteFile rf) {
    	//TODO if it is a large file then scp file & readLocalFile
    	if (doesExist(rf)) {
    		return SshUtil.runCommandGetResults(rf, "cat " + rf.getFullPath());
    	}
    	return null;
    }
    
    
    // ----------------------------------------------------------
    // ----- backupRemoteFile -----------------------------------
    // ----------------------------------------------------------
    
    /**
     * Backs up a remote file fn.txt as fn.txt.bak
     *
     */
    public static void backup(RemoteFile rf) {
        sLogger.debug("backupRemoteFile");
        String mBakFile = rf.getFilename() + ".bak";
        
        StringBuffer mCmd = new StringBuffer();
        mCmd.append("cd " + rf.getDir());
        mCmd.append("; cp " + rf.getFilename());
        mCmd.append(" " + mBakFile);
        mCmd.append("; gzip -dc " + mBakFile);
        SshUtil.runCommand(rf, mCmd.toString());
    }
    

    
    
    // ----------------------------------------------------------
    // ----- clearRemoteFile ------------------------------------
    // ----------------------------------------------------------

    /**
     * 
     */
    public static void clear(RemoteFile rf) {
        //System.out.println("Enter clearRemoteFile(" + user + "," + host + "," + path + "," + file);        

        
        String mCmd="cd " + rf.getDir() + "; echo \"\" > " + rf.getFilename();
        sLogger.debug("mCmd=" + mCmd);        
        String str = SshUtil.runCommandGetResult(rf, mCmd);
        //System.out.println("Exit clearRemoteFile(" + user + "," + host + "," + path + "," + file);        
    }

    // ----------------------------------------------------------
    // ----- getRemoteFileLineCount -----------------------------
    // ----------------------------------------------------------

    
    /**
     * RemoteFile rf
     */
    public static int getLineCount(RemoteFile rf) {
        if (!doesExist(rf)) {
            //DebugUtil.addDebugInfo("getRemoteFileLineCount(): file does not exist, returning -1. Host:" + host + ", user:" + user + ", path:" + path + ", file:" + file);
            return -1;
        }
        
        String mCmd="cd " + rf.getDir() + "; wc -l " + rf.getFilename() + " | cut -d' ' -f1";
        sLogger.debug("mCmd=" + mCmd);        
        String str = SshUtil.runCommandGetResult(rf, mCmd);
        sLogger.debug("getRemoteFileSize(): cmd=" + mCmd + ", result=" + str);
        Integer mInt = new Integer(str);
        return mInt.intValue();
    }

    // ----------------------------------------------------------
    // ----- touchRemoteFile ------------------------------------
    // ----------------------------------------------------------

    
    /**
     * Upudates the time-stamp on a file. Creates it if doesn't exist
     */
    public static void touch(RemoteFile rf) {
        String mCmd="cd " + rf.getDir() + "; touch " + rf.getFilename();
        sLogger.debug("mCmd=" + mCmd);        
        SshUtil.runCommand(rf, mCmd);
    }

    // ----------------------------------------------------------
    // ----- deleteRemoteFile -----------------------------------
    // ----------------------------------------------------------

    /**
     * 
     */
    public static void delete(RemoteFile rf) {
        String mCmd="cd " + rf.getDir() + "; rm " + rf.getFilename() + " 2>stderr.txt; rm stderr.txt";
        sLogger.debug("mCmd=" + mCmd);        
        SshUtil.runCommand(rf, mCmd);
    }

    // ----------------------------------------------------------
    // ----- deleteAndTouchRemoteFile ---------------------------
    // ----------------------------------------------------------

    /**
     * 
     */
    public static void deleteAndTouch(RemoteFile rf) {
        delete(rf);
        touch(rf);
    }

    // ----------------------------------------------------------
    // ----- appendRemoteFile -----------------------------------
    // ----------------------------------------------------------

    /**
     * 
     */
    public static void append(RemoteFile rf, String data) {
        String mCmd="cd " + rf.getDir() + "; echo \"" + data +  "\" >>" + rf.getFilename();
        sLogger.debug("mCmd=" + mCmd);        
        SshUtil.runCommand(rf, mCmd);
    }


    // ----------------------------------------------------------
    // ----- egrep ----------------------------------------------
    // ----------------------------------------------------------

    /**
     * 
     */
    public static List<String> egrep(RemoteFile rf, String includeStr, String excludeStr) {
    	return egrep(rf,includeStr,excludeStr,10);
    }
    /**
     * 
     */
    public static List<String> egrep(RemoteFile rf, String includeStr, String excludeStr, int maxSize) {
    	return egrep(rf,
    			Arrays.asList(includeStr),
    			Arrays.asList(excludeStr),
    			maxSize);
    }

    
    /**
     * 
     */
    public static List<String> egrep(RemoteFile rf, List<String> includeList, List<String> excludeList) {
    	return egrep(rf,includeList,excludeList,10);
    }

    /**
     * 
     * @param rf
     * @param includeList
     * @param excludeList
     * @param maxSize
     * @return
     */
    public static List<String> egrep(RemoteFile rf, List<String> includeList, List<String> excludeList, int maxSize) {
    	if (rf == null || ListUtil.isListNullOrEmpty(includeList)) {
    		return null;
    	}
    	
        StringBuffer mCmd = new StringBuffer();
        mCmd.append("cd " + rf.getDir());
        mCmd.append("; egrep '" + ListUtil.getPipeDelimitedStr(includeList) + "' " + rf.getFilename());
        if (!ListUtil.isListNullOrEmpty(excludeList)) {
        	mCmd.append(" | grep -ve ");
            mCmd.append(ListUtil.getDelimitedStr(excludeList, " -ve "));        	
        }
        mCmd.append(" | head -" + maxSize);
        return SshUtil.runCommandGetResults(rf, mCmd.toString());
    }
    

    /**
     * 
     */
    public static List<String> egrep(RemoteFile rf, List<String> includeList) {
    	return egrep(rf,includeList,10);
    }

    /**
     * 
     * @param rf
     * @param includeList
     * @param excludeList
     * @param maxSize
     * @return
     */
    public static List<String> egrep(RemoteFile rf, List<String> includeList, int maxSize) {
    	return egrep(rf,includeList,null,maxSize);
    }
    
    
    // ----------------------------------------------------------
    // ----- grep -----------------------------------------------
    // ----------------------------------------------------------


    /**
     * 
     */
    public static List<String> grep(RemoteFile rf, String str) {
    	return grep(rf,str,10);
    }

    /**
     * 
     * @param rf
     * @param includeList
     * @param excludeList
     * @param maxSize
     * @return
     */
    public static List<String> grep(RemoteFile rf, String str, int maxSize) {
    	return egrep(rf,Arrays.asList(str),null,maxSize);
    }

    
    
    // ----------------------------------------------------------
    // ----- getErrorsWarningsExceptions ------------------------
    // ----------------------------------------------------------

    public static List<String> getErrorsWarningsExceptions(RemoteFile rf) {
    	return getErrorsWarningsExceptions(rf,10);
    }
    
    public static List<String> getErrorsWarningsExceptions(RemoteFile rf, int maxSize) {
    	List<String> mList = Arrays.asList("ERROR","WARN","Exception");
    	return egrep(rf,mList,maxSize);
    }

    
    // ----------------------------------------------------------
    // ----- getRemoteFileTimeStamp -------------
    // ----------------------------------------------------------

    
    /**
     * 
     */
    public static String getTimeStamp(RemoteFile rf) {
        //
        // init the command
        StringBuffer mCmd = new StringBuffer();
        
        //
        mCmd.append("ls --full-time " + rf.getDir() + rf.getFilename());
        //mCmd.append("| sed 's/  / /g'");    //remove extra spaces
        mCmd.append("| cut -d' ' -f7,8");
        sLogger.debug("mCmd=" + mCmd.toString());

        //save debug info
        //DebugUtil.addDebugInfo("getRemoteFileTimeStamp() cmd=" + mCmd.toString());
//        System.out.println("getRemoteFileTimeStamp() cmd=" + mCmd.toString());
        
        //
        //invoke command
        String mTime = SshUtil.runCommandGetResult(rf, mCmd.toString());
        
        //save debug info
        //DebugUtil.addDebugInfo("getRemoteFileTimeStamp() file=" + file + ", time=" + mTime);
//        System.out.println("getRemoteFileTimeStamp() time=" + mTime);
        
        return mTime;
    }
    
    

    // ------------------------------------------------------------------------
    // ----- copy source file to dest file ------------------------------------
    // ------------------------------------------------------------------------

    /**
     * source & dest files on same remote connection
     */
    public static void copy(RemoteFile source, RemoteFile dest) {
    	StringBuffer cmd = new StringBuffer();
    	cmd.append("cp " + source.getDir() + source.getFilename());
    	cmd.append(" " + dest.getDir() + dest.getFilename());
    	SshUtil.runCommand(source, cmd.toString());
    }
    
    
    // ------------------------------------------------------------------------
    // ----- replace entries in a file using sed ------------------------------
    // ------------------------------------------------------------------------

    
    /**
     * Replaces all occurrences in RemoteFile of origText with newText 
     * 
     * Runs this sed command:
     * 		sed 's/foo/bar/g' filename.txt > tmp.txt; mv tmp.txt filename.txt
     */
    public static void replaceText(RemoteFile rf, String origText, String newText) {
    	StringBuffer cmd = new StringBuffer();
    	cmd.append("cd " + rf.getDir());
    	cmd.append("; sed 's/" + origText + "/" + newText + "/g' " + rf.getFilename());
    	cmd.append(" > tmp.txt; mv tmp.txt " + rf.getFilename());
    	SshUtil.runCommand(rf, cmd.toString());
    	
    }
    
    
    // ------------------------------------------------------------------------
    // ----- unzip a remote zip file ------------------------------------------
    // ------------------------------------------------------------------------

    /**
     * Supports *.zip files
     */
    public static void unzip(RemoteFile rf) {
    	if (rf.getFilename().toLowerCase().endsWith(".zip")) {
    		String cmd = "cd " + rf.getDir() + "; unzip " + rf.getFilename();
    		SshUtil.runCommand(rf, cmd);
    		return;
    	}
    	sLogger.warn("File type not supported for unzipping: " + rf.getFilename());
    }

    // ------------------------------------------------------------------------
    // ----- key/value pairs --------------------------------------------------
    // ------------------------------------------------------------------------

    /**
     * Returns map of all key/value pairs in the file, ie
     * 		foo=bar
     * 
     * Ignores anything commented out, ie
     * 		#goo=moo
     * 
     * Also ignores anything with multiple or zero "=" chars
     */
    public static Map<String,String> getKeyValuePairs(RemoteFile rf) {    	
    	//grep for lines that contain "="
    	//but exclude any that are commented out with "#"
    	List<String> mLines = egrep(rf, "=", "#");    	
    	if (mLines==null || mLines.size()==0) {
    		return null;
    	}
    	
    	// loop entries
    	Map<String,String> mKeyValuePairs = null;
    	for (String bLine : mLines) {
    		String[] bParts = bLine.split("="); 
    		
    		if (bParts==null || bParts.length!=2) {
    			//skip this line, improper format
    		} else {
    			// only init map if we know there are valid key/value pairs
    			if (mKeyValuePairs == null) {
    				mKeyValuePairs = new HashMap<String,String>();
    			}
    			
    			String bKey = bParts[0].trim();
    			String bValue = bParts[1].trim();
    			mKeyValuePairs.put(bKey, bValue);
    		}
    		
    	}

    	return mKeyValuePairs;
    }
    
    
    /**
     * 
     * @param rf
     * @param key
     * @return
     */
    public static String getKeysValue(RemoteFile rf, String key) {
    	Map<String,String> mKeyValuePairs = getKeyValuePairs(rf);
    	
    	if (mKeyValuePairs==null || mKeyValuePairs.size()==0) {
    		return null;
    		
    	} else if (mKeyValuePairs.get(key) == null) {
    		sLogger.error("key(" + key + ") does not exist in " + rf.getFilename());
    		return null;
    	}
    	return mKeyValuePairs.get(key);
    }

    
    /**
     * 
     * @param rf
     * @param key
     * @return
     */
    public static List<String> getKeysValueAsList(RemoteFile rf, String key) {
    	return getKeysValueAsList(rf,key,",");
    }
    public static List<String> getKeysValueAsList(RemoteFile rf, String key, String delim) {
    	String mValue = getKeysValue(rf,key);
    	String[] values = mValue.split(delim);
    	return Arrays.asList(values);
    }

    /**
     * 
     * @param rf
     * @param key
     * @return
     */
    public static boolean doesKeyExist(RemoteFile rf, String key) {
    	String mCurrValue = getKeysValue(rf,key);
    	if (mCurrValue==null || mCurrValue.length()==0) {
    		return false;
    	}
    	return true;
    }
    
    /**
     */
    public static void updateKeysValue(RemoteFile rf, String key, String newValue) {

    	if (!doesKeyExist(rf,key)) {
    		sLogger.error("Key(" + key + ") does not exist in " + rf.getFilename());
    		return;
    	}

    	// find the current setting for this key
    	// need exact syntax so a sed command can be used
    	List<String> mEntries = egrep(rf, key, "#"); //find the exact line entry, exclude commented out lines
    	
    	//make sure there is exactly 1 entry
    	if (mEntries==null || mEntries.size()==0) {
    		sLogger.error("Grep for Key(" + key + ") does not exist in " + rf.getFilename());
    		return;
    	} else if (1 < mEntries.size()) {
    		sLogger.error("More than one entry found for key(" + key + ")");
    		return;
    	}
    	
    	String mOldLine = mEntries.get(0);
    	String mNewLine = key + " = " + newValue;
    	RemoteFileUtil.replaceText(rf, mOldLine, mNewLine);    	
    }

    

}
