package org.qatools.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.qatools.local.LocalDir;
import org.qatools.local.LocalFile;
import org.qatools.remote.RemoteDir;
import org.qatools.remote.RemoteFile;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UIKeyboardInteractive;
import com.jcraft.jsch.UserInfo;

public class ScpUtil extends QAToolsUtil {

	// -------------------------------------------------------------
	// ----- local to remote ---------------------------------------
	// -------------------------------------------------------------

	/**
	 * 
	 * @param lf
	 * @param rd
	 */
	public static void copyLocalFileToRemoteDir(LocalFile lf, RemoteDir rd) {
		//System.out.println("Enter copyLocalFileToRemoteDir()");
		copyLocalFileToRemoteFile(lf,new RemoteFile(rd,lf.getFilename()));
	}
	
	/**
	 * 
	 * @param lf
	 * @param rd
	 */
	public static void copyLocalFileToRemoteFile(LocalFile lf, RemoteFile rf) {
		//System.out.println("Enter copyLocalFileToRemoteFile()");
        FileInputStream fis = null;
        try {

        	//password
        	sRemotePassword = rf.getPassword();
        	
            String lfile = lf.getDir() + lf.getFilename();
            String user = rf.getUser();
            String host = rf.getHost();
            String rfile = rf.getDir() + rf.getFilename();

//    		System.out.println("sRemotePassword = " + sRemotePassword);
//    		System.out.println("user = " + user);
//    		System.out.println("host = " + host);
//    		System.out.println("rfile = " + rfile);
//    		System.out.println("lfile = " + lfile);

            
            JSch jsch = new JSch();
            Session session = jsch.getSession(user, host, 22);

            // username and password will be given via UserInfo interface.
            UserInfo ui = new MyUserInfo();
            session.setUserInfo(ui);
            session.connect();

            // exec 'scp -t rfile' remotely
            String command = "scp -p -t " + rfile;
            Channel channel = session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);

            // get I/O streams for remote scp
            OutputStream out = channel.getOutputStream();
            InputStream in = channel.getInputStream();

            channel.connect();

            if (checkAck(in) != 0) {
                System.exit(0);
            }

            // send "C0644 filesize filename", where filename should not include '/'
            long filesize = (new File(lfile)).length();
            command = "C0644 " + filesize + " ";
            if (lfile.lastIndexOf('/') > 0) {
                command += lfile.substring(lfile.lastIndexOf('/') + 1);
            } else {
                command += lfile;
            }
            command += "\n";
            out.write(command.getBytes());
            out.flush();
            if (checkAck(in) != 0) {
                System.exit(0);
            }

            // send a content of lfile
            fis = new FileInputStream(lfile);
            byte[] buf = new byte[1024];
            while (true) {
                int len = fis.read(buf, 0, buf.length);
                if (len <= 0) {
                    break;
                }
                out.write(buf, 0, len); // out.flush();
            }
            fis.close();
            fis = null;
            // send '\0'
            buf[0] = 0;
            out.write(buf, 0, 1);
            out.flush();
            if (checkAck(in) != 0) {
                System.exit(0);
            }
            out.close();

            channel.disconnect();
            session.disconnect();

        } catch (Exception e) {
            try {
                if (fis != null) {
                    fis.close();
                }
            } catch (Exception ee) {
            }
        }
	}
	
	
	// -------------------------------------------------------------
	// ----- remote to local ---------------------------------------
	// -------------------------------------------------------------

	/**
	 * 
	 */
	public static void copyRemoteFileToLocalDir(RemoteFile rf, LocalDir ld) {
		//System.out.println("Enter copyRemoteFileToLocalDir()");
		copyRemoteFileToLocalFile(rf, new LocalFile(ld,rf.getFilename()));
	}

	/**
	 * 
	 * @param rf
	 * @param lf
	 */
	public static void copyRemoteFileToLocalFile(RemoteFile rf, LocalFile lf) {
		//System.out.println("Enter copyRemoteFileToLocalFile()");
		
        FileOutputStream fos = null;
        try {
        	//password
        	sRemotePassword = rf.getPassword();

        	String user = rf.getUser();
            String host = rf.getHost();
            String rfile = rf.getDir() + rf.getFilename();
            String lfile = lf.getDir() + lf.getFilename();

//    		System.out.println("sRemotePassword = " + sRemotePassword);
//    		System.out.println("user = " + user);
//    		System.out.println("host = " + host);
//    		System.out.println("rfile = " + rfile);
//    		System.out.println("lfile = " + lfile);
            
            
            String prefix = null;
            if (new File(lfile).isDirectory()) {
                prefix = lfile + File.separator;
            }

            JSch jsch = new JSch();
            Session session = jsch.getSession(user, host, 22);

            // username and password will be given via UserInfo interface.
            UserInfo ui = new MyUserInfo();
            session.setUserInfo(ui);
            session.connect();

            // exec 'scp -f rfile' remotely
            String command = "scp -f " + rfile;
            Channel channel = session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);

            // get I/O streams for remote scp
            OutputStream out = channel.getOutputStream();
            InputStream in = channel.getInputStream();

            channel.connect();

            byte[] buf = new byte[1024];

            // send '\0'
            buf[0] = 0;
            out.write(buf, 0, 1);
            out.flush();

            while (true) {
                int c = checkAck(in);
                if (c != 'C') {
                    break;
                }

                // read '0644 '
                in.read(buf, 0, 5);

                long filesize = 0L;
                while (true) {
                    if (in.read(buf, 0, 1) < 0) {
                        // error
                        break;
                    }
                    if (buf[0] == ' ') {
                        break;
                    }
                    filesize = filesize * 10L + (buf[0] - '0');
                }

                String file = null;
                for (int i = 0;; i++) {
                    in.read(buf, i, 1);
                    if (buf[i] == (byte) 0x0a) {
                        file = new String(buf, 0, i);
                        break;
                    }
                }

                //System.out.println("filesize="+filesize+", file="+file);

                // send '\0'
                buf[0] = 0;
                out.write(buf, 0, 1);
                out.flush();

                // read a content of lfile
                fos = new FileOutputStream(prefix == null ? lfile : prefix + file);
                int foo;
                while (true) {
                    if (buf.length < filesize) {
                        foo = buf.length;
                    } else {
                        foo = (int) filesize;
                    }
                    foo = in.read(buf, 0, foo);
                    if (foo < 0) {
                        // error
                        break;
                    }
                    fos.write(buf, 0, foo);
                    filesize -= foo;
                    if (filesize == 0L) {
                        break;
                    }
                }
                fos.close();
                fos = null;

                if (checkAck(in) != 0) {
                	System.out.println("checkAck(in) != 0 ... System.exit(0)");
                    System.exit(0);
                }

                // send '\0'
                buf[0] = 0;
                out.write(buf, 0, 1);
                out.flush();
            }

            session.disconnect();

        } catch (Exception e) {
            try {
                if (fos != null) {
                    fos.close();
                }
            } catch (Exception ee) {
            }
        }
	}
	
    // -----------------------------------------------------------------------------
    // ----- checkAck --------------------------------------------------------------
    // -----------------------------------------------------------------------------

	/**
	 * 
	 */
    static int checkAck(InputStream in) throws IOException {
        int b = in.read();
        // b may be 0 for success,
        // 1 for error,
        // 2 for fatal error,
        // -1
        if (b == 0) {
            return b;
        }
        if (b == -1) {
            return b;
        }

        if ((b == 1) || (b == 2)) {
            StringBuffer sb = new StringBuffer();
            int c;
            do {
                c = in.read();
                sb.append((char) c);
            } while (c != '\n');
            if (b == 1) { // error
                System.out.print(sb.toString());
            }
            if (b == 2) { // fatal error
                System.out.print(sb.toString());
            }
        }
        return b;
    }

    // ---------------------------------------------------------
    // ----- inner class with password info --------------------
    // ---------------------------------------------------------
    
    private static String sRemotePassword = null;   //used to over-ride the default password

    public static class MyUserInfo implements UserInfo, UIKeyboardInteractive {
        public String getPassword() {
                return sRemotePassword;
        }

        public boolean promptYesNo(String str) {
            return true;
        }

        public String getPassphrase() {
            return null;
        }

        public boolean promptPassphrase(String message) {
            return true;
        }

        public boolean promptPassword(String message) {
            return true;
        }

        public void showMessage(String message) {}

        public String[] promptKeyboardInteractive(String arg0, String arg1, String arg2, String[] arg3, boolean[] arg4) {
            return null;
        }
    }
	
}
