package org.qatools.util;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.qatools.remote.RemoteConn;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.UIKeyboardInteractive;
import com.jcraft.jsch.UserInfo;

public class SshUtil extends QAToolsUtil {
    private static Logger sLogger = Logger.getLogger(SshUtil.class);

    /** Default listen port for SSH daemon */
    private static final int SSH_PORT = 22;
    private static final int BUFFER_SIZE = 1024; // bytes
    private static final int PAUSE_DURATION = 1000; // milliseconds


	// -------------------------------------------------------------
	// ----- runCommand --------------------------------------------
	// -------------------------------------------------------------

    public static void runCommand(RemoteConn rc, String command) {
    	doCommandGetResults(rc,command);
    }
    public static String runCommandGetResult(RemoteConn rc, String command) {
    	return doCommandGetResult(rc,command);
    }
    public static List<String> runCommandGetResults(RemoteConn rc, String command) {
    	return doCommandGetResults(rc,command);
    }

    
	// -------------------------------------------------------------
	// ----- doCommand ---------------------------------------------
	// -------------------------------------------------------------

    /**
     * 
     */
    private static String doCommandGetResult(RemoteConn rc, String command) {
    	List<String> mResults = doCommandGetResults(rc,command);
    	
    	if (mResults==null || mResults.size()==0) {
    		return null;
    		
    	} else if (1 < mResults.size()) {
    		sLogger.warn("Expected 1 result but got " + mResults.size() + " results.");
    		return mResults.get(0);
    		
    	} else {
    		return mResults.get(0);
    	}
    }

    
    /**
     * run SSH shell command
     */
    private static List<String> doCommandGetResults(RemoteConn rc, String command) {
        //System.out.println("Enter doCommandGetResults()");
        boolean mPass=true;
        StringBuffer mResult = new StringBuffer("");
        try {
            JSch jsch = new JSch();
            Session session = jsch.getSession(rc.getUser(), rc.getHost(), SSH_PORT);

            // username and password will be given via UserInfo interface.
            SSHUserInfo ui = new SSHUserInfo();
            ui.setPassword(rc.getPassword());
            session.setUserInfo(ui);
            session.connect();

            Channel channel = session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);
            channel.setXForwarding(true);
            channel.setInputStream(System.in);
            ((ChannelExec) channel).setErrStream(System.err);
            InputStream in = channel.getInputStream();
            channel.connect();

            byte[] tmp = new byte[BUFFER_SIZE];
            while (true) {
                while (in.available() > 0) {
                    int i = in.read(tmp, 0, BUFFER_SIZE);
                    if (i < 0) {
                        break;
                    }
                    mResult.append(new String(tmp, 0, i));
                }
                if (channel.isEOF() && (in.available() <= 0)) {
                    break;
                }
                try {
                    Thread.sleep(PAUSE_DURATION);
                } catch (Exception ee) {
                    String bMsg = "sshUtil.runCommand(" + rc.getHost() + ", " + rc.getUser() + ", " + rc.getPassword() + "," + command + ") ee=" + ee;
                    System.out.println(bMsg);
                    //DebugUtil.addDebugInfo(bMsg);  
                    mPass=false;
                }
            }
            channel.disconnect();
            session.disconnect();
        } catch (Exception e) {
            String bMsg = "sshUtil.runCommand(" + rc.getHost() + ", " + rc.getUser() + ", " + rc.getPassword() + "," + command + ") e=" + e;
            System.out.println(bMsg);
            //DebugUtil.addDebugInfo(bMsg);            
            mPass=false;
        }
        //System.out.println("Exit runCommand()");
        if (mPass) {
        	return parseResultsForNewlines(mResult.toString().trim());
        }
        return null;
    }

	// -------------------------------------------------------------
	// ----- parseResultsForNewlines -------------------------------
	// -------------------------------------------------------------

    /**
     * 
     */
    private static List<String> parseResultsForNewlines(String s) {
    	String[] lines = s.split("\n");
    	
    	if (lines==null || lines.length==0) {
    		return null;
    	}
    	
    	List<String> mResults = new ArrayList<String>();
    	for (int i=0; i<lines.length; i++) {
    		mResults.add(lines[i]);
    	}
    	return mResults;
    }
    
    
	// -------------------------------------------------------------
	// ----- SSHUserInfo -------------------------------------------
	// -------------------------------------------------------------

    private static class SSHUserInfo implements UserInfo, UIKeyboardInteractive {
        private String password = null;

        public String getPassword() {
            return password;
        }

        public void setPassword(String passwd) {
            this.password = passwd;
        }

        public boolean promptYesNo(String str) {
            return true;
        }

        public String getPassphrase() {
            return null;
        }

        public boolean promptPassphrase(String message) {
            return true;
        }

        public boolean promptPassword(String message) {
            if (password != null) {
                return true;
            }
            return false;
        }

        public void showMessage(String message) {}
        
        public String[] promptKeyboardInteractive(String arg0, String arg1, String arg2, String[] arg3, boolean[] arg4) {
            return null;
        }

    }
	
}
